/*
This file is part of [aconnect] library. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

#include "client.hpp"

#include "util.hpp"

namespace aconnect 
{
	TcpClient::TcpClient (const ip_addr_type host, const port_type port, bool autoConnect) throw (socket_error) : 
				_port (port),
				_domain (AF_INET),
				_socket (INVALID_SOCKET)
	{
		assert (host);
		memcpy ((void*) _host, (void*) host, sizeof(ip_addr_type));

		if (autoConnect)
			connectToHost();
	}

	void TcpClient::connectToHost() throw (socket_error)
	{
		if (_socket != INVALID_SOCKET)
			throw socket_error (_socket, "Client socket already connected");

		_socket = aconnect::util::createSocket();

		struct sockaddr_in dest;
		util::zeroMemory (&dest, sizeof( dest ));

		dest.sin_family = _domain;
		dest.sin_port = htons ( _port );
		util::writeIpAddress (dest.sin_addr, _host);

		if ( 0 != connect(_socket, (sockaddr*) &dest, sizeof(dest)) )
            throw socket_error (_socket, "Could not connect socket");
	}

	//////////////////////////////////////////////////////////////////////////

	void TcpClient::write (string_constptr data, size_t dataSize) throw (socket_error)
	{
		aconnect::util::writeToSocket(_socket, data, dataSize);
	}

	string TcpClient::readToEndMark (string_constptr mark) throw (socket_error)
	{
		EndMarkSocketStateCheck check(mark);
		return aconnect::util::readFromSocket(_socket, check);
	}

	string TcpClient::readPacket (size_t packetSize) throw (socket_error)
	{
		ResponseSizeSocketStateCheck check(packetSize);
		
		return aconnect::util::readFromSocket(_socket, check);
	}

	void TcpClient::close () throw (socket_error)
	{
		util::closeSocket(_socket);
		_socket = INVALID_SOCKET;
	}

}