/*
This file is part of [aconnect] library. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

#ifndef ACONNECT_TYPES_H
#define ACONNECT_TYPES_H

#include "config.hpp"
#include <string>
#include <cstring>
#include <cstdarg>
#include <cstdio>


#ifdef WIN32
#	ifndef WIN32_LEAN_AND_MEAN 
#		define WIN32_LEAN_AND_MEAN 
#	endif
#	include <windows.h>
#	include <winsock2.h>
#	pragma comment (lib, "ws2_32.lib")
#endif

#	ifdef max
#		undef max
#	endif
#	ifdef min
#		undef min
#	endif

#if !defined(WIN32)
	typedef int errno_t;
#endif

namespace aconnect
{
	#ifndef NULL
	#	define NULL 0
	#endif

	#if !defined(WIN32)
	#	include <cctype>	// for tolower
	#	if !defined (INVALID_SOCKET)
	#		define INVALID_SOCKET (~0)
	#	endif
	
	#	if !defined (SOCKET_ERROR)
	#		define SOCKET_ERROR (-1)
	#	endif
	#	
	#	define __interface struct
	#
	#endif  // !(WIN32)
	
	
	// useful string types
	// now works only with MBCS
	using	std::string;		

	typedef const std::string&				string_constref;
	typedef std::string&					string_ref;
	typedef std::string::pointer			string_ptr;
	typedef std::string::const_pointer		string_constptr;
	
	typedef std::string::value_type			char_type;
	typedef const char_type					string_constant[];

	typedef std::wstring					wide_string;

	// socket's related types
	typedef int port_type;
	typedef unsigned char byte_type;
	typedef byte_type ip_addr_type[4];

	namespace Tristate
	{
		enum TristateEnum
		{
			Undefined = -1,
			False = 0,
			True = 1
		};
	};
	

#if !defined(WIN32)
	typedef int err_type;
#else
	typedef errno_t err_type;
#endif

#ifdef WIN32
	typedef SOCKET socket_type;
	typedef int socket_length_type;	
	typedef HMODULE library_handle_type;
	typedef HANDLE handle_type;
	typedef HANDLE process_id_type;

#else
	typedef int socket_type;
	typedef socklen_t socket_length_type;
	typedef void* library_handle_type;
	typedef void* handle_type;
	typedef unsigned long process_id_type;
#endif

	
// some safe functions definition
#if !defined(WIN32)
#	define vsprintf_s(buff, size, format, arg) vsprintf(buff, format, arg)
#	define mkgmtime(tm_) timegm(tm_)
	//extern size_t strlen (const char*);
	
	inline int caseInsensitiveCharCompareN(char_type a, char_type b) {	
		return (std::tolower(a) - std::tolower(b)); 
	}
	
	inline int stricmp (string_constptr str1, string_constptr str2) 
	{
		size_t len1 = strlen(str1);
		size_t len2 = strlen(str2);

		if (len1 > len2)
			return 1;
		if (len2 > len1)
			return -1;

		for (size_t ndx=0; ndx<len1; ++ndx) {
			int n = caseInsensitiveCharCompareN (str1[ndx], str2[ndx]);
			if (0 != n)
				return (n > 0 ? 1 : -1);
		}
		return 0;
	}  
  	
#elif defined(WIN32)
#	define stricmp(s1_, s2_)  _stricmp(s1_, s2_)
#	define mkgmtime(tm_) _mkgmtime(tm_)
// AKM: not safe
#	define snprintf _snprintf 
#endif

#define FORMAT_VA_MESSAGE_EX(FormatStr, FormatVarName, OutStr, BuffSize)	\
	va_list args__; \
	va_start (args__, FormatVarName); \
	char buff__[BuffSize]; \
	int cnt__ = vsprintf_s (buff__, BuffSize, FormatStr, args__); \
	aconnect::string OutStr;\
	if (cnt__ > 0)\
	OutStr.append (buff__, cnt__);\
	va_end(args__);

#define FORMAT_VA_MESSAGE(FormatStr, OutStr) FORMAT_VA_MESSAGE_EX(FormatStr, FormatStr, OutStr, 0x1000)

#define FORMAT_VA_MESSAGE_DIRECT(FormatStr, FormatVarName, OutStr) FORMAT_VA_MESSAGE_EX(FormatStr, FormatVarName, OutStr, 0x1000)

#define ARRAY_SIZE(Array) (sizeof(Array)/sizeof(Array[0]))
};

#endif // ACONNECT_TYPES_H
