/*
This file is part of [ahttp] library. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

//////////////////////////////////////////////////////////////////////////
//		HTTP header loading socket state checker
//////////////////////////////////////////////////////////////////////////


class HttpHeaderReadCheck : public aconnect::SocketStateCheck
{
public:

	HttpHeaderReadCheck (HttpRequestHeader	*header, aconnect::Server *server, 
			bool isKeepAliveConnect, long keepAliveTimeoutSec) 
		: _headerRead (false), 
		_header(header), 
		_server (server), 
		_headerSize(0),
		_isKeepAliveConnect (isKeepAliveConnect)
	{
		assert (_header);
		assert (_server);
		// setup HTTP server Keep-Alive timeout
		_readTimeout.tv_sec = keepAliveTimeoutSec;
		_readTimeout.tv_usec = 0;

		FD_ZERO ( &_readSet);
	}

	virtual void prepare (aconnect::socket_type s) {
		if (_isKeepAliveConnect ) {
			FD_ZERO ( &_readSet);
			FD_SET (s, &_readSet);
		}
	};

	virtual bool isDataAvailable (aconnect::socket_type s) {
		if (!_isKeepAliveConnect )
			return true;

		int selectRes = select ( (int) s + 1, &_readSet, NULL, NULL, &_readTimeout);
		if (0  == selectRes || SOCKET_ERROR == selectRes)	
		{
			aconnect::util::closeSocket (s);
			setConnectionWasClosed(true);
		
			return false;
		}
		return true; // there is data to read
	}

	virtual bool readCompleted (aconnect::socket_type s, string_constref data) {
		using namespace aconnect;
		static const string EndMarkStandard = "\r\n\r\n";
		static const string EndMarkShort = "\n\n";

		if (!_headerRead) {
			size_t pos, endMarkLen;

			if ((pos = data.find (EndMarkStandard)) != string::npos) {
				endMarkLen = EndMarkStandard.length();
				_headerRead = true;

			} else if ((pos = data.find (EndMarkShort)) != string::npos) {
				endMarkLen = EndMarkShort.length();
				_headerRead = true;
			} 
			
			if (_headerRead) {
				if (_header->load (data.substr (0, pos)))
					_headerSize = pos + endMarkLen;
				else
					return true; // process Bad Request 
			}
		}

		return _headerRead;
	}

	inline size_t headerSize() const				{	return _headerSize;	}

protected:
	bool				_headerRead;
	HttpRequestHeader	*_header;
	aconnect::Server	*_server;
	size_t				_headerSize;

	// Keep-Alive
	bool				_isKeepAliveConnect;	
	fd_set				_readSet;
	timeval				_readTimeout;
};
