/*
This file is part of [ahttp] library. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

#ifndef AHTTP_MESSAGES_H
#define AHTTP_MESSAGES_H

namespace ahttp 
{ 
	string_constref HttpServerSettings::getMessage (string_constref key) 
		const throw (std::range_error) 
	{
		aconnect::str2str_map::const_iterator it = _messages.find(key);
		if (it == _messages.end())
			throw std::range_error ("Message not found: " + key);
 
		return it->second;
	}

	void HttpServerSettings::loadMessage (string_constref key, string_constptr value)
	{
		if (value)
			_messages [key] = value;
	}

	void HttpServerSettings::initMessages()
	{
		_messages["ErrorUndefined"] = 
			"Detailed error description is not available.";

		// 301-303
		_messages["ErrorDocumentMoved"] = 
			"This document may be found <a HREF=\"%s\">here</a>.";
		
		_messages["Error400"] = 
			"The request could not be understood by the server due to malformed syntax.<br />%s";

		_messages["Error401"] = 
			"You are not authorized to view this page.";

		_messages["Error403_BrowseContent"] = 
			"This Virtual Directory does not allow contents to be listed.";

		_messages["Error403_AccessDenied"] = 
			"Access to filesystem object is denied.";

		_messages["Error404"] = 
			"The requested page cannot be found: \"%s\".";

		_messages["Error405"] = 
			"The requested HTTP method is not allowed for the resource: %s.<br /> Allowed methods: %s.";

		_messages["Error406_CharsetNotAllowed"] = 
			"The response's content charset is not allowed by client.";

		_messages["Error413"] = 
			"Request size is too large to be processed by server.";

		_messages["Error500"] = 
			"Internal server error.<hr />%s";

		_messages["ServerError_FileInsteadDirectory"] = 
			"File path retrieved instead of directory";

		_messages["Error503"] = 
			"The server is currently unable to handle the request due to a temporary overloading or maintenance of the server.";

		_messages["Error501_MethodNotImplemented"] = 
			"Requested method (%s) is not implemented, try GET/POST/HEAD.";

		_messages["Error500_RequestNotLoaded"] = 
			"HTTP request was not loaded correctly.";

		_messages["Error500_ProcessingFailed"] =
			"Request processing failed for <i>%s</i>, see details in server log.";

		_messages["MessageFormat"] = 
			"<html><head><title>%s</title> \
			<style> BODY { padding: 10px; margin: 10px; font: 10pt Tahoma, Arial; color: #000;}\
			H1 {color: #cc0000; font: 14pt Tahoma, Arial; font-weight: bold; } \
			HR {height:1px; border: 1px solid #333; color: #333;} \
			TABLE {font-size: 100%%;}</style> \
			</head><body><h1>%s</h1>%s</body></html>";

		_messages["MessageFormatInline"] = "<hr /><b style=\"color:#cc0000;\">%s</b><br/>%s";

	}
}

#endif // AHTTP_MESSAGES_H

