/*
This file is part of [ahttpserver] project. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

namespace AHttpInternal
{
	using namespace System::Threading;
	using namespace System::Web;
	using namespace System::Web::Hosting;
	using namespace System::Diagnostics;
	
	using namespace System::Runtime::Remoting::Lifetime;
	using namespace System::Security;
	using namespace System::Security::Permissions;
	using namespace System::Security::Policy;
	using namespace System::Reflection;
	
	using namespace Microsoft::Win32; // registry
	
	using namespace AHttp;
	using namespace ahttp::strings::ServerVariables;

	public ref class Application : AspNetServer, MarshalByRefObject
	{
	
	public:
		static Application^ Server = gcnew Application();
		String^ InstallPath;
		Dictionary< int,  AHttp::AspNetHost^> ^Hosts;
		


	public:
		Application () : 
			InstallPath (nullptr),
			Hosts (gcnew Dictionary<int,  AHttp::AspNetHost^>())
		{
		}


		virtual void ReloadHost (int number, ApplicationShutdownReason shutdownReasonreason)
		{
			if (0 == Globals::HttpServer || Globals::HttpServer->isStopped())
				return;

			const ahttp::DirectorySettings* settings = Globals::HttpServer->GlobalSettings()->getDirSettings(number);
			if (settings)
			{
				String^ reason = shutdownReasonreason.ToString();

				Globals::HttpServer->Log()->info ("ASPNET: application unloaded by Hosting Environment, directory: %s, reason: %s",
						settings->name.c_str(),
						getUnmanagedString (reason).c_str());
				
				boost::mutex::scoped_lock lock(Globals::LoadMutex);
				Globals::DestroyCondition.notify_one();
			}
		}

		// critical section is not needed
		AHttp::AspNetHost^ InitHost (const ahttp::DirectorySettings& dirSettings, bool isAutoReaload) 
		{
			System::Threading::Monitor::Enter (AHttpInternal::Application::typeid);
			
			if (String::IsNullOrEmpty (InstallPath))
				InstallPath = SetupInstallPathAndConfigureAspNetIfNeeded ();

            System::Threading::Monitor::Exit (AHttpInternal::Application::typeid);
            
			String^ realAppPath = getManagedString (dirSettings.realPath);
			String^ virtualAppPath = getManagedString (dirSettings.virtualPath);
			
			AHttp::AspNetHost^ host = nullptr;

            try {
                System::Threading::Monitor::Enter (AHttpInternal::Application::Server);
                
                if (!Hosts->ContainsKey(dirSettings.number))
                {
        			host = (AHttp::AspNetHost^) ApplicationHost::CreateApplicationHost (
        				AHttp::AspNetHost::typeid, 
        				virtualAppPath,
        				realAppPath);
        					
        			host->Init(InstallPath->ToLower(), this, dirSettings.number);
					
					host->GetDomain();
					
					//host->Ping();

    				Hosts->Add ( dirSettings.number, host);
                
                } else {
                    host = Hosts[dirSettings.number];
                }
            
            } finally {
                System::Threading::Monitor::Exit (AHttpInternal::Application::Server);
            }

			if (isAutoReaload) 
			{
				aconnect::string reloadUrl = aconnect::util::getItemFromMap (dirSettings.customParams, Globals::Param_AspReloadUrl);
		                
				if (!reloadUrl.empty())
				{
					try {
						String^ response = host->ProcessSimpleRequest (getManagedString (reloadUrl));
					
					} catch (Exception^ ex) {
						Globals::HttpServer->Log()->info ("ASPNET: application reload target call failed, reason: %s",
							getUnmanagedString (ex->ToString()).c_str());
					}
				}
			}

			return host;
			
		}

		// Copied from Cassini - only MS guys know what does it mean
		String^ SetupInstallPathAndConfigureAspNetIfNeeded() 
		{
            // If ASP.NET was never registered on this machine, the registry 
            // needs to be patched up for System.Web.dll to find aspnet_isapi.dll
            //
            // If HKLM\Microsoft\ASP.NET key is missing, this will be added
            //      (adjusted for the correct directory and version number
            // [HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\ASP.NET]
            //      "RootVer"="1.0.3514.0"
            // [HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\ASP.NET\1.0.3514.0]
            //      "Path"="E:\WINDOWS\Microsoft.NET\Framework\v1.0.3514"
            //      "DllFullPath"="E:\WINDOWS\Microsoft.NET\Framework\v1.0.3514\aspnet_isapi.dll"

            String^ AspNetKeyName = L"Software\\Microsoft\\ASP.NET";

            RegistryKey^ aspNetKey = nullptr;
            RegistryKey^ aspNetVersionKey = nullptr;
            RegistryKey^ frameworkKey = nullptr;

			String^ installPath;

            
            try {
				// get the version corresponding to System.Web.Dll currently loaded
                FileVersionInfo^  aspNetVersionInfo = FileVersionInfo::GetVersionInfo(HttpRuntime::typeid->Module->FullyQualifiedName); 

				String^ aspNetVersion = String::Format(L"{0}.{1}.{2}.{3}", 
					aspNetVersionInfo->FileMajorPart, 
					aspNetVersionInfo->FileMinorPart, 
					aspNetVersionInfo->FileBuildPart, 
					aspNetVersionInfo->FilePrivatePart);
                String^ aspNetVersionKeyName = AspNetKeyName + L"\\" + aspNetVersion;

                // non 1.0 names should have 0 QFE in the registry
                if (!aspNetVersion->StartsWith(L"1.0."))
                    aspNetVersionKeyName = aspNetVersionKeyName->Substring(0, aspNetVersionKeyName->LastIndexOf(L'.')+1) + L"0";

                // check if the subkey with version number already exists
				aspNetVersionKey = Registry::LocalMachine->OpenSubKey(aspNetVersionKeyName);

                if (aspNetVersionKey != nullptr) 
				{
                    // already created -- just get the path
                    return (String^) aspNetVersionKey->GetValue(L"Path");
                }
                else 
				{
                    // open/create the ASP.NET key
					aspNetKey = Registry::LocalMachine->OpenSubKey(AspNetKeyName);
                    if (aspNetKey == nullptr) {
						aspNetKey = Registry::LocalMachine->CreateSubKey(AspNetKeyName);
                        // add RootVer if creating
                        aspNetKey->SetValue(L"RootVer", aspNetVersion);
                    }

                    // version dir name is almost version: "1.0.3514.0" -> "v1.0.3514"
                    String^ versionDirName = L"v" + aspNetVersion->Substring(0, aspNetVersion->LastIndexOf(L'.'));

                    // install directory from "InstallRoot" under ".NETFramework" key
					frameworkKey = Registry::LocalMachine->OpenSubKey(L"Software\\Microsoft\\.NETFramework");
                    
					String^ rootDir = (String^) frameworkKey->GetValue(L"InstallRoot");
                    if (rootDir->EndsWith(L"\\"))
                        rootDir = rootDir->Substring(0, rootDir->Length - 1);

                    // create the version subkey
					aspNetVersionKey = Registry::LocalMachine->CreateSubKey(aspNetVersionKeyName);

                    // install path
                    installPath = rootDir + L"\\" + versionDirName;

                    // set path and dllfullpath
                    aspNetVersionKey->SetValue("Path", installPath);
                    aspNetVersionKey->SetValue("DllFullPath", installPath + "\\aspnet_isapi.dll");

                }
            }
            
			catch (System::Exception^) {
				if (String::IsNullOrEmpty (installPath) )
					throw gcnew ApplicationException (L"ASP.NET installation path cannot be loaded");
            }
            
			finally {
                if (aspNetVersionKey != nullptr)
                    aspNetVersionKey->Close();
                
				if (aspNetKey != nullptr)
                    aspNetKey->Close();
                
				if (frameworkKey != nullptr)
                    frameworkKey->Close();
            }

			return installPath;
        }
	

	};

	
}