/*
This file is part of [ahttpserver] project. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

//////////////////////////////////////////////////////////////////////////

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(startWsgiResponse_Overloads, startWsgiResponse, 2, 3);

BOOST_PYTHON_MODULE(python_handler)
{
	using namespace boost::python;

	typedef std::map <aconnect::string, ahttp::UploadFileInfo> files_map;

	class_< aconnect::str_vector >("StrVector")
		.def(vector_indexing_suite <aconnect::str_vector, true >() );
	
	class_< aconnect::str2str_map >("Str2StrMap")
		.def(map_indexing_suite <aconnect::str2str_map, true>() );
	
	class_< files_map >("UploadedFilesMap")
		.def(map_indexing_suite <files_map>() );

	class_<ahttp::UploadFileInfo>("UploadFileInfo", no_init)
		.add_property	("name", &ahttp::UploadFileInfo::name)
		.add_property	("file_name", &ahttp::UploadFileInfo::fileName)
		.add_property	("content_type", &ahttp::UploadFileInfo::contentType)
		.add_property	("upload_path", &ahttp::UploadFileInfo::uploadPath)
		.add_property	("size", &ahttp::UploadFileInfo::fileSize)
		;

	class_<TracebackLoaderWrapper,
		detail::not_specified, 
		detail::not_specified, 
		boost::noncopyable>("TracebackLoader", no_init)
		.def("write", &TracebackLoaderWrapper::write)
		.def_readonly("content", &TracebackLoaderWrapper::content );
	
	class_<RequestHeaderWrapper, 
		detail::not_specified, 
		detail::not_specified, 
		boost::noncopyable> ("HttpRequestHeader", init<ahttp::HttpRequestHeader *>())
		.def			("__len__", &RequestHeaderWrapper::getLength)
		.def			("__getitem__", &RequestHeaderWrapper::getHeader)
		.def			("has_header", &RequestHeaderWrapper::hasHeader)

		.add_property("items", make_function ( &RequestHeaderWrapper::items,
			return_value_policy<copy_const_reference>() ))

		.add_property	("method", &RequestHeaderWrapper::requestMethod)
		.add_property	("http_ver_high", &RequestHeaderWrapper::requestHttpVerHigh)
		.add_property	("http_ver_low", &RequestHeaderWrapper::requestHttpVerLow)
		
		// selected headers
		.add_property	("user_agent", &RequestHeaderWrapper::userAgent)
		;
	
	class_<RequestWrapper, 
		detail::not_specified, 
		detail::not_specified, 
		boost::noncopyable> ("HttpRequest", init<ahttp::HttpContext *>())

		.add_property("get", make_function (&RequestWrapper::getParameters,
				return_value_policy<copy_const_reference>()))
		.add_property("post", make_function (&RequestWrapper::postParameters,
				return_value_policy<copy_const_reference>()))
		.add_property("cookies", make_function (&RequestWrapper::cookies,
				return_value_policy<copy_const_reference>()))
		.add_property("files", make_function (&RequestWrapper::files,
				return_value_policy<copy_const_reference>()))
		.def ("__getitem__", &RequestWrapper::param)
		
		.def ("raw_read", &RequestWrapper::read)
		.def ("is_read", &RequestWrapper::isRead)
		.def ("process", &RequestWrapper::processRequest)
		
		// WSGI
		.def ("read", &RequestWrapper::read)
		.def ("readline", &RequestWrapper::readline)
		.def ("readlines", &RequestWrapper::readlines)

		;

	class_<HttpContextWrapper, 
		detail::not_specified, 
		detail::not_specified, 
		boost::noncopyable> ("HttpContext", init<ahttp::HttpContext *>())
	
	.add_property	("path", &HttpContextWrapper::path)
	.add_property	("virtual_path", &HttpContextWrapper::virtualPath)
	.add_property	("initial_virtual_path", &HttpContextWrapper::initialVirtualPath)
	.add_property	("script_path", &HttpContextWrapper::scriptPath)
	
	// client info
	.add_property	("client_ip", &HttpContextWrapper::clientIpAddr)
	.add_property	("client_port", &HttpContextWrapper::clientPort)
	.add_property	("server_port", &HttpContextWrapper::serverPort)

	// request properties
	.add_property	("request_header", &HttpContextWrapper::RequestHeader)
	.add_property	("request", &HttpContextWrapper::Request)
	.def			("is_client_connected", &HttpContextWrapper::isClientConnected)
	.def			("map_path", &HttpContextWrapper::mapPath)
	.def			("get_server_var", &HttpContextWrapper::getServerVariable)
		
	// response modification
	.def			("write", &HttpContextWrapper::write)
	.def			("write_escaped", &HttpContextWrapper::writeEscaped)
	.def			("flush", &HttpContextWrapper::flush)
	.def			("set_content_type", &HttpContextWrapper::setContentType)
	.def			("set_utf8_html_response", &HttpContextWrapper::setUtf8Html)
	.def			("process_error", &HttpContextWrapper::processServerError)
	.def			("set_header", &HttpContextWrapper::setResponseHeader)
	.add_property	("status", &HttpContextWrapper::getResponseStatus, &HttpContextWrapper::setResponseStatus)

	// WSGI
	.def			("__call__",  &HttpContextWrapper::startWsgiResponse, startWsgiResponse_Overloads())
	;

	// WSGI 'write' callable
	class_<ResponseWriterWrapper, 
		detail::not_specified, 
		detail::not_specified, 
		boost::noncopyable> ("HttpResponseWriter", init<ahttp::HttpContext *>())
	.def			("__call__",  &ResponseWriterWrapper::write)
	;



	def ("escape_html", (aconnect::string (*)(aconnect::string_constref)) aconnect::util::escapeHtml);
	def ("decode_url", aconnect::util::decodeUrl);
	def ("encode_url_part", aconnect::util::encodeUrlPart);
}



