/*
This file is part of [ahttpserver] project. 

Author: Artem Kustikov (kustikoff[at]tut.by)


This code is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this code.

Permission is granted to anyone to use this code for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this code must not be misrepresented; you must
not claim that you wrote the original code. If you use this
code in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original code.

3. This notice may not be removed or altered from any source
distribution.
*/

#ifdef WIN32

namespace AHttp
{
	System::String^ getManagedString (aconnect::string_constref str) {
		return gcnew System::String (aconnect::util::getWideString (str.c_str()).c_str());
	}


	aconnect::string getUnmanagedString (System::String^ str) throw (std::runtime_error)
	{
		if (String::IsNullOrEmpty(str))
			return aconnect::string();

		// From MSDN:	
		// Pin memory so GC can't move it while native function is called
		pin_ptr<const wchar_t> wch = PtrToStringChars(str);
		size_t convertedChars = 0;
		size_t  sizeInBytes = ((str->Length + 32 ) * 2);
		
		boost::scoped_array<char> buff  (new char [sizeInBytes]); 
		
		errno_t err = wcstombs_s(&convertedChars, 
						buff.get(), sizeInBytes,
						wch, sizeInBytes);
		if (err != 0)
			throw std::runtime_error ("Couldn't convert String^ (wide to MB) ");

		return aconnect::string (buff.get(), convertedChars - 1);
	}
}

#endif // WIN32