
// each test module could contain no more then one 'main' file with init function defined
// alternatively you could define init function yourself

#include <boost/test/unit_test.hpp>

#include "aconnect/util.hpp"
#include "aconnect/server.hpp"
#include "aconnect/client.hpp"



//==================================================================//

aconnect::string_constant KeepAliveMessage ("Hello, I am alive!");
const aconnect::port_type ServerPort = 8888;
const aconnect::ip_addr_type LocalhostAddress = {127, 0, 0, 1};

void echoThreadProc (const aconnect::ClientInfo& client) 
{ 
	using namespace aconnect;
    static EndMarkSocketStateCheck check; 
    string request = client.getRequest (check); 
    request.erase ( request.find(check.endMark()) );

    // write response 
    client.writeResponse ("Echo: " + request); 
}

void keepAliveThreadProc (const aconnect::ClientInfo& client) 
{ 
    client.writeResponse (KeepAliveMessage); 
}


//==================================================================//


BOOST_AUTO_TEST_CASE( aconnect_server_simple_run )
{
    using namespace aconnect;

    // global server instance
    Server  server;
    Initializer init;

    server.init (ServerPort, keepAliveThreadProc);

	// test default settings
	BOOST_CHECK (server.settings().socketReadTimeout == 60);
	BOOST_CHECK (server.settings().socketWriteTimeout == 60);
	BOOST_CHECK (server.settings().enablePooling == true);

	server.start(); // start in child thread

	BOOST_CHECK (server.currentWorkersCount() == 0);

	// connect to server

	for (int ndx=0; ndx<100; ++ndx) 
	{
		TcpClient client (LocalhostAddress, ServerPort);

		string response = client.readPacket ( strlen(KeepAliveMessage) );
		BOOST_REQUIRE ( strcmp(response.c_str(), KeepAliveMessage) == 0 );

		client.close();
	}
	
    server.stop (true);
}


BOOST_AUTO_TEST_CASE( aconnect_server_start_stop )
{
    using namespace aconnect;

    // global server instance
    Server  server;
    Initializer init;

    server.init (ServerPort, keepAliveThreadProc);
	server.start(); // start in child thread

	BOOST_CHECK (server.currentWorkersCount() == 0);

	TcpClient client (LocalhostAddress, ServerPort);

	string response = client.readPacket ( strlen(KeepAliveMessage) );
	BOOST_REQUIRE ( strcmp(response.c_str(), KeepAliveMessage) == 0 );
	
	client.close();
	
    server.stop (true);
	
	bool serverStopped = false;
	try
	{
		client.connectToHost();
	
	} catch (socket_error& ex ) {

		BOOST_CHECK (ex.code() == aconnect::network::error::ConnectionRefused);
		client.close(); 
		serverStopped = true;
	}
	BOOST_CHECK_MESSAGE ( serverStopped, "Server must be stopped" );

	// second connection
	server.start(); // start in child thread

	
	client.connectToHost();
	response = client.readPacket ( strlen(KeepAliveMessage) );

	BOOST_REQUIRE ( strcmp(response.c_str(), KeepAliveMessage) == 0 );
	
	client.close();
	server.stop (true);
}
