
// each test module could contain no more then one 'main' file with init function defined
// alternatively you could define init function yourself

#include <boost/test/unit_test.hpp>
#include <boost/lexical_cast.hpp>

#include <iostream>


#include "aconnect/util.hpp"
#include "aconnect/util.time.hpp"

#include "aconnect/thread_pool.hpp"
#include "aconnect/client.hpp"
#include "aconnect/complex_types.hpp"

//==================================================================//

boost::detail::atomic_count ThreadPool_RunWithSleep_Int_COUNTER(0l);
aconnect::str_stream output;

void threadProcIntSleep (int n)
{
	aconnect::util::sleep (rand() * 500/  RAND_MAX );

	aconnect::string res ("threadProcIntSleep: ");
	res += boost::lexical_cast<aconnect::string> (n);
	res += "\r\n";

	// std::cout << res;
	output << res;
    ++ThreadPool_RunWithSleep_Int_COUNTER;
}

BOOST_AUTO_TEST_CASE( ThreadPool_RunWithSleep_Int )
{
	BOOST_TEST_MESSAGE ("ThreadPool_RunWithSleep_Int");

	using namespace aconnect;

    output.str("");
        
	ThreadPool<int> pool(threadProcIntSleep, 20);
		
	const int Count = 100;
	for (int n=0; n < Count; ++n) {
		aconnect::util::sleep (50);	
		pool.process(n);
	}

	aconnect::util::sleep (1000);	
	pool.wait();
    
    BOOST_REQUIRE_EQUAL (ThreadPool_RunWithSleep_Int_COUNTER, Count);
    
    aconnect::string content = output.str();
    
    BOOST_REQUIRE (content.find ("threadProcIntSleep: 0") != aconnect::string::npos);
    BOOST_REQUIRE (content.find ("threadProcIntSleep: 50") != aconnect::string::npos);
    BOOST_REQUIRE (content.find ("threadProcIntSleep: 99") != aconnect::string::npos);
}

//==================================================================//

boost::detail::atomic_count ThreadPool_Run_Int_COUNTER(0l);

void threadProcInt (int n)
{
	aconnect::util::sleep (rand() * 200/  RAND_MAX );
	++ThreadPool_Run_Int_COUNTER;
}

BOOST_AUTO_TEST_CASE( ThreadPool_Run_Int )
{
	BOOST_TEST_MESSAGE ("ThreadPool_Run_Int");

	using namespace aconnect;
       
	ThreadPool<int> pool(threadProcInt, 20);
	
	const int Count = 30;
	for (int n=0; n < Count; ++n)
		pool.process(n);
	
	aconnect::util::sleep (1000);
	pool.wait();

	BOOST_REQUIRE_MESSAGE (ThreadPool_Run_Int_COUNTER == Count, 
		("Processed only " + boost::lexical_cast<aconnect::string>(ThreadPool_Run_Int_COUNTER)) + " items");
}

//==================================================================//
namespace VolumeTest
{
	boost::detail::atomic_count ThreadProcVolume_COUNTER(0l);
	const int ThreadsCount = 10;
	const int ThreadCallsCount = 25;

	struct VolumeTestItem
	{
		int index;
		boost::thread::id pid;
	};

	void threadProcVolume (VolumeTestItem item)
	{
		//std::cout << (boost::lexical_cast<aconnect::string> (item.pid) + ": " + boost::lexical_cast<aconnect::string> (item.index) + "\r\n");
		++ThreadProcVolume_COUNTER;
		
	}

	aconnect::ThreadPool<VolumeTestItem> VolumeTestPool(threadProcVolume, 50);

	void threadProcVolumeRunner ()
	{
		for (int ndx=0; ndx < VolumeTest::ThreadCallsCount; ++ndx)
		{
			VolumeTestItem item = {ndx, aconnect::util::getCurrentThreadId()};
			VolumeTestPool.process (item);
			aconnect::util::sleep (50);
		}
	};
}

BOOST_AUTO_TEST_CASE( ThreadPool_VolumeTest )
{
	BOOST_TEST_MESSAGE ("ThreadPool_VolumeTest");

	using namespace aconnect;
    	
	boost::thread_group group;
		
	for (int n=0; n < VolumeTest::ThreadsCount; ++n)
	{
		group.create_thread (VolumeTest::threadProcVolumeRunner);
	}

	group.join_all();
	
	aconnect::util::sleep (1000);	
	VolumeTest::VolumeTestPool.wait();

	BOOST_REQUIRE_EQUAL (VolumeTest::ThreadProcVolume_COUNTER, 
		VolumeTest::ThreadsCount * VolumeTest::ThreadCallsCount);
}

//==================================================================//

namespace PriorityTest
{
	const int ItemsCount = 30;
	int Counter = 0;
	std::vector <int> Results;

	boost::mutex Mutex;

	void threadProcPriority_Int (int n)
	{
		using namespace aconnect;
		
		boost::mutex::scoped_lock lock (Mutex);

		Results.push_back (n);
		++Counter;
	}
}

BOOST_AUTO_TEST_CASE( ThreadPoolWithPriority_Int )
{
	BOOST_TEST_MESSAGE ("ThreadPoolWithPriority_Int");

	using namespace aconnect;
       
	ThreadPoolWithPriority<int> pool(PriorityTest::threadProcPriority_Int, 10);
	
	for (int n=0; n < PriorityTest::ItemsCount; ++n)
		pool.addItemToProcess(n);
	
	pool.startProcessing(500);
	pool.wait();

	BOOST_REQUIRE_EQUAL (PriorityTest::Counter, PriorityTest::ItemsCount);
	BOOST_REQUIRE (PriorityTest::Results.front() == (PriorityTest::ItemsCount - 1));

}

//==================================================================//