
// each test module could contain no more then one 'main' file with init function defined
// alternatively you could define init function yourself

#include <boost/test/unit_test.hpp>
#include <boost/lexical_cast.hpp>

#include "aconnect/util.string.hpp"
#include "aconnect/complex_types.hpp"

BOOST_AUTO_TEST_CASE( Parse_Key_Value_Pairs )
{
	using namespace aconnect;

	std::map<string, string> pairs;
	
	std::map<string, string> map1;
	map1["m"] = "8";
	map1["n"] = "9";
	map1["x"] = "78";

	aconnect::util::parseKeyValuePairs ("m=8;n=9;x=78", pairs);
	BOOST_REQUIRE ( pairs == map1 );

	BOOST_REQUIRE ( util::getItemFromMap (pairs, "m") == "8" );
	BOOST_REQUIRE ( util::getItemFromMap (pairs, "x") == "78" );


	std::map<string, string> map2;
	string keys[] = {"abb", "bbb", "cbb", "dbb", "ebb", "last"};
	const int keysCount = ARRAY_SIZE (keys);

	str_stream strToParse;
	string key, value;

	for (int ndx=0; ndx<100; ++ndx) {
		value = boost::lexical_cast<string> (ndx);
		key = keys[ndx % 5] + value;
		
		map2[key] = value;
		if (ndx > 0)
			strToParse << "\r\n";

		strToParse << key << ':' << value;
	}

	aconnect::util::parseKeyValuePairs (strToParse.str(), pairs, ':', "\r\n");
	BOOST_REQUIRE ( pairs == map2 );
}

BOOST_AUTO_TEST_CASE( Wide_String_Test )
{
	using namespace aconnect;

	BOOST_CHECK ( util::getWideString ("test") == L"test" );
	
	char* res = setlocale (LC_CTYPE, ".1251");
	BOOST_CHECK (NULL != res);

	BOOST_CHECK ( util::getWideString ("") == L"" );
	BOOST_CHECK ( util::getWideString ("Some long\r\nlong\r\nlong\r\nlong\r\nlong\r\nlong sequence") == L"Some long\r\nlong\r\nlong\r\nlong\r\nlong\r\nlong sequence" );
}


BOOST_AUTO_TEST_CASE( UTF8_Test )
{
	using namespace aconnect;

	BOOST_CHECK ( util::decodeUrl ("test") == "test" );

	BOOST_CHECK ( util::decodeUrl ("http://localhost:5555/media/84/12.jpg") == "http://localhost:5555/media/84/12.jpg" );
	
	char* res = setlocale (LC_CTYPE, ".1251");
	BOOST_CHECK (NULL != res);

	string converted = util::getLocalizedStringFromUtf8 (util::decodeUrl ("%D1%8D%D0%B9%D0%BD%D1%88%D1%82%D0%B5%D0%B9%D0%BD.jpg"));

	BOOST_CHECK ( converted == ".jpg" );

	
}