
// each test module could contain no more then one 'main' file with init function defined
// alternatively you could define init function yourself

#include <boost/test/unit_test.hpp>
#include <boost/filesystem.hpp>

#include "ahttplib.hpp"
#include "aconnect/util.hpp"

namespace fs = boost::filesystem;
namespace Globals
{
	extern aconnect::string ApplicationDirectory;

	ahttp::HttpServer Server;
	ahttp::HttpServerSettings HttpSettings;
	aconnect::ConsoleLogger Logger (aconnect::Log::Info);

#if defined(_DEBUG)
	aconnect::string PluginName = "handler_fastcgi-d.dll";
#else
	aconnect::string PluginName = "handler_fastcgi.dll";
#endif

}


//==================================================================//

 
BOOST_AUTO_TEST_CASE( Cgi_Application )
{
	using namespace aconnect;

	fs::path serverOutDir = fs::complete( "../../out/plugins", fs::path(Globals::ApplicationDirectory));
	BOOST_CHECK ( fs::exists(serverOutDir) );

	fs::path pluginLibPath = fs::complete( Globals::PluginName, serverOutDir).normalize();
	BOOST_CHECK ( fs::exists(pluginLibPath) );
	BOOST_CHECK ( fs::is_regular_file(pluginLibPath) );

	HMODULE dll = ::LoadLibraryA (pluginLibPath.file_string().c_str());
	BOOST_REQUIRE_MESSAGE ( NULL != dll, "FastCGI handler not found");
	
	FARPROC processRequestFunc = ::GetProcAddress (dll, ahttp::SettingsConstants::ProcessRequestFunName);
	BOOST_CHECK ( NULL != processRequestFunc );

	FARPROC initFunc = ::GetProcAddress (dll, ahttp::SettingsConstants::InitFunName);
	BOOST_CHECK ( NULL != initFunc );

	FARPROC destroyFunc = ::GetProcAddress (dll, ahttp::SettingsConstants::DestroyFunName);
	BOOST_CHECK ( NULL != destroyFunc );

	aconnect::Initializer init; // load WinSockets

	aconnect::str2str_map params;
	params["engine"] = "c:\\PHP\\php-cgi.exe";
	params["mode"] = "tcp";
	params["tcp-listener-port"] = "5444";
	params["check-file-exists"] = "false";

	Globals::HttpSettings.setPort (9999);
	Globals::HttpSettings.setLogger (&Globals::Logger);
	Globals::Server.init (&Globals::HttpSettings);
	
	bool inited = reinterpret_cast<ahttp::init_plugin_function> (initFunc) 
				(params, 1, &Globals::Server);
	
	BOOST_REQUIRE ( inited );

	destroyFunc ();

	::FreeLibrary (dll);
}